@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2023,2024 Thomas E. Dickey                                *
+ * Copyright 2018-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2017,2018 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -30,7 +30,7 @@
 /****************************************************************************
  *  Author: Thomas E. Dickey                    1996-on                     *
  ****************************************************************************/
-/* $Id: test.priv.h,v 1.220 2024/10/05 19:01:00 tom Exp $ */
+/* $Id: test.priv.h,v 1.227 2025/08/23 17:05:52 tom Exp $ */
 
 #ifndef __TEST_PRIV_H
 #define __TEST_PRIV_H 1
@@ -743,16 +743,28 @@ extern int optind;
 #define HELP_KEY_2	KEY_F(1)
 
 /* our "standard" options for getopt, needed for help2man */
+#define OPTS_COMMAND	'c'
+#define OPTS_LOGGING	'l'
 #define OPTS_USAGE	'h'
 #define OPTS_VERSION	'V'
-#define OPTS_COMMON	"hV"
+#define OPTS_COMMON	"c:l:hV"
 #define USAGE_COMMON	\
  "Common options:"\
 ," -h       show this message"\
 ," -V       show version of curses"
 
+/* reserve -c/-l for command/logging */
+#define CASE_COMMON \
+	case OPTS_COMMAND: \
+	case OPTS_LOGGING: \
+	    usage(ch == OPTS_USAGE); \
+	    ExitProgram(EXIT_FAILURE); \
+	case OPTS_VERSION: \
+	    show_version(argv); \
+	    ExitProgram(EXIT_SUCCESS)
+
 #if HAVE_CURSES_VERSION
-#define format_version(buffer, size) strcpy(buffer, curses_version())
+#define format_version(buffer, size) _nc_STRCPY(buffer, curses_version(), size)
 #elif defined(NCURSES_VERSION_MAJOR) && defined(NCURSES_VERSION_MINOR) && defined(NCURSES_VERSION_PATCH)
 #define format_version(buffer, size) \
 	_nc_SPRINTF(buffer, _nc_SLIMIT(size) "ncurses %d.%d.%d", \
@@ -760,7 +772,7 @@ extern int optind;
 		    NCURSES_VERSION_MINOR, \
 		    NCURSES_VERSION_PATCH)
 #else
-#define format_version(buffer, size) strcpy(buffer, "ncurses-examples")
+#define format_version(buffer, size) _nc_STRCPY(buffer, "ncurses-examples", size)
 #endif
 
 #define VERSION_COMMON() \
@@ -852,6 +864,12 @@ extern "C" {
 #define _nc_SLIMIT(n)		/* nothing */
 #endif
 
+/*
+ * To make them easier to find, user-defined capabilities used within ncurses
+ * should be tagged with this macro:
+ */
+#define UserCap(name) #name
+
 /*
  * X/Open Curses does not define the arrays of terminfo/termcap names as SVr4
  * curses did, and some implementations provide them anyway, but undeclared.
@@ -1066,11 +1084,8 @@ extern int TABSIZE;
 #include <sys/time.h>		/* for struct timeval */
 #undef sleep
 #define sleep(n) Sleep((n) * 1000)
-#define SIGHUP  1
-#define SIGKILL 9
-#define getlogin() "username"
 
-#elif defined(EXP_WIN32_DRIVER)
+#else /* !PDCURSES */
 
 #if defined(HAVE_NCURSESW_NCURSES_H)
 #include <ncursesw/nc_win32.h>
@@ -1080,23 +1095,23 @@ extern int TABSIZE;
 #include <nc_win32.h>
 #endif
 
-#else
+#endif /* PDCURSES */
 
-#if defined(HAVE_NCURSESW_NCURSES_H)
-#include <ncursesw/nc_mingw.h>
-#elif defined(HAVE_NCURSES_NCURSES_H)
-#include <ncurses/nc_mingw.h>
-#else
-#include <nc_mingw.h>
+#define getlogin() "username"
+
+#ifndef SIGHUP
+#define SIGHUP  1
 #endif
 
+#ifndef SIGKILL
+#define SIGKILL 9
 #endif
 
 /* conflicts in test/firstlast.c */
 #undef large
 #undef small
 
-#endif
+#endif /* WIN32... */
 
 #ifdef NEED_TIME_H
 #if TIME_WITH_SYS_TIME
